@@ -104,36 +104,36 @@ class Line(BranchParent):
                               device_type=DeviceType.LineDevice)
 
         # line length in km
-        self._length = length
+        self._length = float(length)
 
         # line impedance tolerance
-        self.tolerance = tolerance
+        self.tolerance = float(tolerance)
 
         # short circuit impedance
-        self.r_fault = r_fault
-        self.x_fault = x_fault
-        self.fault_pos = fault_pos
+        self.r_fault = float(r_fault)
+        self.x_fault = float(x_fault)
+        self.fault_pos = float(fault_pos)
 
         # total impedance and admittance in p.u.
-        self.R = r
-        self.X = x
-        self.B = b
+        self.R = float(r)
+        self.X = float(x)
+        self.B = float(b)
 
-        self.R0 = r0
-        self.X0 = x0
-        self.B0 = b0
+        self.R0 = float(r0)
+        self.X0 = float(x0)
+        self.B0 = float(b0)
 
-        self.R2 = r2
-        self.X2 = x2
-        self.B2 = b2
+        self.R2 = float(r2)
+        self.X2 = float(x2)
+        self.B2 = float(b2)
 
         # Conductor base and operating temperatures in ºC
-        self.temp_base = temp_base
-        self.temp_oper = temp_oper
+        self.temp_base = float(temp_base)
+        self.temp_oper = float(temp_oper)
         self._temp_oper_prof = Profile(default_value=temp_oper, data_type=float)
 
         # Conductor thermal constant (1/ºC)
-        self.alpha = alpha
+        self.alpha = float(alpha)
 
         # type template
         self.template: Union[OverheadLineType, SequenceLineType, UndergroundLineType] = template
@@ -394,33 +394,43 @@ class Line(BranchParent):
         elm.temperature_prof = self.temp_oper_prof
         return elm
 
-    def fill_design_properties(self, r_ohm, x_ohm, c_nf, length, Imax, freq, Sbase):
+    def fill_design_properties(self, r_ohm, x_ohm, c_nf, length, Imax, freq, Sbase, apply_to_profile: bool = True,):
         """
         Fill R, X, B from not-in-per-unit parameters
-        :param r_ohm: Resistance per km in OHM
-        :param x_ohm: Reactance per km in OHM
-        :param c_nf: Capacitance per km in nF
+        :param r_ohm: Resistance per km in OHM/km
+        :param x_ohm: Reactance per km in OHM/km
+        :param c_nf: Capacitance per km in nF/km
         :param length: lenght in kn
         :param Imax: Maximum current in kA
         :param freq: System frequency in Hz
         :param Sbase: Base power in MVA (take always 100 MVA)
+        :param apply_to_profile: modify the profile is checked
         """
-        R = r_ohm * length
-        X = x_ohm * length
-        # impedance = 1 / (2 * pi * f * c), susceptance = (2 * pi * f * c)
-        B = (2 * np.pi * freq * c_nf * 1e-9) * length
+        r_ohm_total = r_ohm * length
+        x_ohm_total = x_ohm * length
+        # impedance = 1 / (2 * pi * f * c),
+        # susceptance = (2 * pi * f * c)
+        b_siemens_total = (2 * np.pi * freq * c_nf * 1e-9) * length
 
         Vf = self.get_max_bus_nominal_voltage()
 
         Zbase = (Vf * Vf) / Sbase
         Ybase = 1.0 / Zbase
 
-        self.R = np.round(R / Zbase, 6)
-        self.X = np.round(X / Zbase, 6)
-        self.B = np.round(B / Ybase, 6)
-        self.rate = np.round(Imax * Vf * 1.73205080757, 6)  # nominal power in MVA = kA * kV * sqrt(3)
+        self.R = np.round(r_ohm_total / Zbase, 6)
+        self.X = np.round(x_ohm_total / Zbase, 6)
+        self.B = np.round(b_siemens_total / Ybase, 6)
+
+        old_rate = float(self.rate)
+        new_rate = np.round(Imax * Vf * 1.73205080757, 6)  # nominal power in MVA = kA * kV * sqrt(3)
+
+        self.rate = new_rate
         self.length = length
 
+        if apply_to_profile:
+            prof_old = self.rate_prof.toarray()
+            self.rate_prof.set(prof_old * new_rate / old_rate)
+
     def get_virtual_taps(self) -> Tuple[float, float]:
         """
         Get the branch virtual taps
