@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from GridCalEngine.IO.base.units import UnitMultiplier, UnitSymbol
-from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.generating_unit import GeneratingUnit
+from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.generating_unit import GeneratingUnit
 from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile
 
 
@@ -23,14 +23,4 @@ class SolarGeneratingUnit(GeneratingUnit):
 	def __init__(self, rdfid='', tpe='SolarGeneratingUnit'):
 		GeneratingUnit.__init__(self, rdfid, tpe)
 
-		from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.solar_power_plant import SolarPowerPlant
-		self.SolarPowerPlant: SolarPowerPlant | None = None
 
-		self.register_property(
-			name='SolarPowerPlant',
-			class_type=SolarPowerPlant,
-			multiplier=UnitMultiplier.none,
-			unit=UnitSymbol.none,
-			description='''A solar power plant may have solar generating units.''',
-			profiles=[]
-		)
