@@ -15,22 +15,31 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from GridCalEngine.IO.base.units import UnitMultiplier, UnitSymbol
-from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.equipment import Equipment
-from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile
+from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.equipment import Equipment
+from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile, UnitSymbol
 
 
 class DCConductingEquipment(Equipment):
 	def __init__(self, rdfid='', tpe='DCConductingEquipment'):
 		Equipment.__init__(self, rdfid, tpe)
 
-		from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.dc_terminal import DCTerminal
+		self.ratedUdc: float = None
+		from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.dc_terminal import DCTerminal
 		self.DCTerminals: DCTerminal | None = None
 
+		self.register_property(
+			name='ratedUdc',
+			class_type=float,
+			multiplier=UnitMultiplier.k,
+			unit=UnitSymbol.V,
+			description='''Electrical voltage, can be both AC and DC.''',
+			profiles=[]
+		)
 		self.register_property(
 			name='DCTerminals',
 			class_type=DCTerminal,
 			multiplier=UnitMultiplier.none,
 			unit=UnitSymbol.none,
-			description='''None''',
+			description='''A DC conducting equipment has DC terminals.''',
 			profiles=[]
 		)
