@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from GridCalEngine.IO.base.units import UnitMultiplier, UnitSymbol
-from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.generating_unit import GeneratingUnit
+from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.generating_unit import GeneratingUnit
 from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile, WindGenUnitKind
 
 
@@ -24,12 +24,22 @@ class WindGeneratingUnit(GeneratingUnit):
 		GeneratingUnit.__init__(self, rdfid, tpe)
 
 		self.windGenUnitType: WindGenUnitKind = None
+		from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.wind_power_plant import WindPowerPlant
+		self.WindPowerPlant: WindPowerPlant | None = None
 
 		self.register_property(
 			name='windGenUnitType',
 			class_type=WindGenUnitKind,
 			multiplier=UnitMultiplier.none,
 			unit=UnitSymbol.none,
-			description='''The kind of wind generating unit''',
+			description='''The kind of wind generating unit.''',
+			profiles=[]
+		)
+		self.register_property(
+			name='WindPowerPlant',
+			class_type=WindPowerPlant,
+			multiplier=UnitMultiplier.none,
+			unit=UnitSymbol.none,
+			description='''A wind power plant may have wind generating units.''',
 			profiles=[]
 		)
