@@ -1,5 +1,5 @@
 # GridCal
-# Copyright (C) 2015 - 2023 Santiago Peñate Vera
+# Copyright (C) 2015 - 2024 Santiago Peñate Vera
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,21 +14,23 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+from __future__ import annotations
 import os
 import json
 
 from collections.abc import Callable
-from typing import Union, List
+from typing import Union, List, Any, Dict, TYPE_CHECKING
+
+from GridCalEngine.IO.cim.cgmes.gridcal_to_cgmes import gridcal_to_cgmes, create_cgmes_headers
+from GridCalEngine.IO.cim.cgmes.cgmes_export import CimExporter
+from GridCalEngine.IO.cim.cgmes.cgmes_data_parser import CgmesDataParser
 from GridCalEngine.basic_structures import Logger
 from GridCalEngine.data_logger import DataLogger
-
 from GridCalEngine.IO.gridcal.json_parser import save_json_file_v3
-from GridCalEngine.IO.cim.cim16.cim_parser import CIMExport
 from GridCalEngine.IO.gridcal.excel_interface import save_excel, load_from_xls, interpret_excel_v3, interprete_excel_v2
-from GridCalEngine.IO.gridcal.pack_unpack import create_data_frames, data_frames_to_circuit
-from GridCalEngine.IO.matpower.matpower_parser import interpret_data_v1
+from GridCalEngine.IO.gridcal.pack_unpack import gather_model_as_data_frames, parse_gridcal_data, gather_model_as_jsons
+from GridCalEngine.IO.matpower.matpower_parser import interpret_data_v1, parse_matpower_file
 from GridCalEngine.IO.dgs.dgs_parser import dgs_to_circuit
-from GridCalEngine.IO.matpower.matpower_parser import parse_matpower_file
 from GridCalEngine.IO.others.dpx_parser import load_dpx
 from GridCalEngine.IO.others.ipa_parser import load_iPA
 from GridCalEngine.IO.gridcal.json_parser import parse_json, parse_json_data_v2, parse_json_data_v3
@@ -36,15 +38,85 @@ from GridCalEngine.IO.raw.raw_parser_writer import read_raw, write_raw
 from GridCalEngine.IO.raw.raw_to_gridcal import psse_to_gridcal
 from GridCalEngine.IO.raw.gridcal_to_raw import gridcal_to_raw
 from GridCalEngine.IO.epc.epc_parser import PowerWorldParser
-# from GridCalEngine.IO.cim.cim16.cim_parser import CIMImport
-from GridCalEngine.IO.cim.cgmes_2_4_15.cgmes_circuit import CgmesCircuit
-from GridCalEngine.IO.cim.cgmes_2_4_15.cgmes_to_gridcal import cgmes_to_gridcal
-from GridCalEngine.IO.gridcal.zip_interface import save_data_frames_to_zip, get_frames_from_zip
+from GridCalEngine.IO.cim.cim16.cim_parser import CIMImport, CIMExport
+from GridCalEngine.IO.cim.cgmes.cgmes_circuit import CgmesCircuit, is_valid_cgmes
+from GridCalEngine.IO.cim.cgmes.cgmes_to_gridcal import cgmes_to_gridcal
+from GridCalEngine.IO.gridcal.zip_interface import save_gridcal_data_to_zip, get_frames_from_zip
 from GridCalEngine.IO.gridcal.sqlite_interface import save_data_frames_to_sqlite, open_data_frames_from_sqlite
 from GridCalEngine.IO.gridcal.h5_interface import save_h5, open_h5
 from GridCalEngine.IO.raw.rawx_parser_writer import parse_rawx, write_rawx
 from GridCalEngine.IO.others.pypsa_parser import parse_netcdf, parse_hdf5
-from GridCalEngine.Core.Devices.multi_circuit import MultiCircuit
+from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile
+from GridCalEngine.Devices.multi_circuit import MultiCircuit
+from GridCalEngine.Simulations.results_template import DriverToSave
+from GridCalEngine.Simulations.PowerFlow.power_flow_results import PowerFlowResults
+from GridCalEngine.enumerations import CGMESVersions, SimulationTypes
+
+if TYPE_CHECKING:
+    from GridCalEngine.Simulations.types import DRIVER_OBJECTS
+
+
+class FileSavingOptions:
+    """
+    This class is to store the extra stuff that needs to be passed to save more complex files
+    """
+
+    def __init__(self,
+                 cgmes_boundary_set: str = "",
+                 simulation_drivers: List[DRIVER_OBJECTS] = None,
+                 sessions_data: List[DriverToSave] = None,
+                 dictionary_of_json_files: Dict[str, Dict[str, Any]] = None,
+                 cgmes_version: CGMESVersions = CGMESVersions.v2_4_15,
+                 cgmes_profiles: Union[None, List[cgmesProfile]] = None,
+                 one_file_per_profile: bool = False):
+        """
+        Constructor
+        :param cgmes_boundary_set: CGMES boundary set zip file path
+        :param simulation_drivers: List of Simulation Drivers
+        :param sessions_data: List of sessions_data
+        :param dictionary_of_json_files: Dictionary of json files
+        :param cgmes_version: Version to use with CGMES
+        :param cgmes_profiles: CGMES profile list to export
+        :param one_file_per_profile: use one file per profile?
+        """
+
+        self.cgmes_version: CGMESVersions = cgmes_version
+
+        self.cgmes_boundary_set: str = cgmes_boundary_set
+
+        self.simulation_drivers: List[DRIVER_OBJECTS] = simulation_drivers if simulation_drivers else list()
+
+        self.sessions_data: List[DriverToSave] = sessions_data if sessions_data else list()
+
+        self.dictionary_of_json_files = dictionary_of_json_files if dictionary_of_json_files else dict()
+
+        # File type description as it appears in the file saving dialogue i.e. GridCal zip (*.gridcal)
+        self.type_selected: str = ""
+
+        # CGMES profile list
+        self.cgmes_profiles = cgmes_profiles if cgmes_profiles is not None else [cgmesProfile.EQ,
+                                                                                 cgmesProfile.OP,
+                                                                                 cgmesProfile.SC,
+                                                                                 cgmesProfile.TP,
+                                                                                 cgmesProfile.SV,
+                                                                                 cgmesProfile.SSH,
+                                                                                 cgmesProfile.DY,
+                                                                                 cgmesProfile.DL,
+                                                                                 cgmesProfile.GL]
+
+        # use one file per profile?
+        self.one_file_per_profile = one_file_per_profile
+
+    def get_power_flow_results(self) -> Union[None, PowerFlowResults]:
+        """
+        Try to extract the power flow results
+        :return: None or PowerFlowResults
+        """
+        for data in self.sessions_data:
+            if data.tpe == SimulationTypes.PowerFlow_run:
+                return data.results
+
+        return None
 
 
 class FileOpen:
@@ -52,10 +124,13 @@ class FileOpen:
     File open interface
     """
 
-    def __init__(self, file_name: Union[str, List[str]]):
+    def __init__(self,
+                 file_name: Union[str, List[str]],
+                 previous_circuit: Union[MultiCircuit, None] = None):
         """
         File open handler
         :param file_name: name of the file
+        :param previous_circuit: previous circuit
         """
         self.file_name: Union[str, List[str]] = file_name
 
@@ -69,15 +144,20 @@ class FileOpen:
 
         self.cgmes_logger = DataLogger()
 
-        if type(self.file_name) == str:
+        self._previous_circuit = previous_circuit
+
+        if isinstance(self.file_name, str):
             if not os.path.exists(self.file_name):
                 raise Exception("File not found :( \n{}".format(self.file_name))
-        elif type(self.file_name) == list:
+        elif isinstance(self.file_name, list):
             for fname in self.file_name:
                 if not os.path.exists(fname):
                     raise Exception("File not found :( \n{}".format(fname))
+        else:
+            raise Exception("file_name type not supported :( \n{}".format(self.file_name))
 
-    def open(self, text_func: Union[None, Callable] = None,
+    def open(self,
+             text_func: Union[None, Callable] = None,
              progress_func: Union[None, Callable] = None) -> Union[MultiCircuit, None]:
         """
         Load GridCal compatible file
@@ -92,12 +172,14 @@ class FileOpen:
             for f in self.file_name:
                 _, file_extension = os.path.splitext(f)
                 if file_extension.lower() not in ['.xml', '.zip']:
-                    raise Exception('Loading multiple files that are not XML/Zip (xml or zip is for CIM or CGMES)')
+                    raise ValueError('Loading multiple files that are not XML/Zip (xml or zip is for CIM or CGMES)')
 
-            self.cgmes_circuit = CgmesCircuit(text_func=text_func, progress_func=progress_func, logger=self.cgmes_logger)
-            self.cgmes_circuit.parse_files(files=self.file_name)
+            data_parser = CgmesDataParser(text_func=text_func, progress_func=progress_func, logger=self.cgmes_logger)
+            data_parser.load_files(files=self.file_name)
+            self.cgmes_circuit = CgmesCircuit(cgmes_version=data_parser.cgmes_version, text_func=text_func,
+                                              progress_func=progress_func, logger=self.cgmes_logger)
+            self.cgmes_circuit.parse_files(data_parser=data_parser)
             self.circuit = cgmes_to_gridcal(cgmes_model=self.cgmes_circuit, logger=self.cgmes_logger)
-
         else:
 
             if os.path.exists(self.file_name):
@@ -121,15 +203,17 @@ class FileOpen:
 
                     elif data_dictionary['version'] == 4.0:
                         if data_dictionary is not None:
-                            self.circuit = data_frames_to_circuit(data_dictionary, logger=self.logger)
+                            self.circuit = parse_gridcal_data(data=data_dictionary,
+                                                              text_func=text_func,
+                                                              progress_func=progress_func,
+                                                              logger=self.logger)
                         else:
                             self.logger.add("Error while reading the file :(")
                             return None
-
                     else:
                         self.logger.add('The file could not be processed')
 
-                elif file_extension.lower() == '.gridcal':
+                elif file_extension.lower() in ['.gridcal']:
 
                     # open file content
                     data_dictionary, self.json_files = get_frames_from_zip(self.file_name,
@@ -138,7 +222,28 @@ class FileOpen:
                                                                            logger=self.logger)
                     # interpret file content
                     if data_dictionary is not None:
-                        self.circuit = data_frames_to_circuit(data_dictionary, logger=self.logger)
+                        self.circuit = parse_gridcal_data(data=data_dictionary,
+                                                          text_func=text_func,
+                                                          progress_func=progress_func,
+                                                          logger=self.logger)
+                    else:
+                        self.logger.add("Error while reading the file :(")
+                        return None
+
+                elif file_extension.lower() in ['.dgridcal']:
+
+                    # open file content
+                    data_dictionary, self.json_files = get_frames_from_zip(self.file_name,
+                                                                           text_func=text_func,
+                                                                           progress_func=progress_func,
+                                                                           logger=self.logger)
+                    # interpret file content
+                    if data_dictionary is not None:
+                        self.circuit = parse_gridcal_data(data=data_dictionary,
+                                                          previous_circuit=self._previous_circuit,
+                                                          text_func=text_func,
+                                                          progress_func=progress_func,
+                                                          logger=self.logger)
                     else:
                         self.logger.add("Error while reading the file :(")
                         return None
@@ -151,7 +256,7 @@ class FileOpen:
                                                                    progress_func=progress_func)
                     # interpret file content
                     if data_dictionary is not None:
-                        self.circuit = data_frames_to_circuit(data_dictionary, logger=self.logger)
+                        self.circuit = parse_gridcal_data(data_dictionary, logger=self.logger)
                     else:
                         self.logger.add("Error while reading the file :(")
                         return None
@@ -223,11 +328,20 @@ class FileOpen:
                     self.logger += parser.logger
 
                 elif file_extension.lower() in ['.xml', '.zip']:
-                    self.cgmes_circuit = CgmesCircuit(text_func=text_func,
-                                                      progress_func=progress_func,
-                                                      logger=self.cgmes_logger)
-                    self.cgmes_circuit.parse_files(files=[self.file_name])
-                    self.circuit = cgmes_to_gridcal(cgmes_model=self.cgmes_circuit, logger=self.cgmes_logger)
+                    data_parser = CgmesDataParser(text_func=text_func, progress_func=progress_func,
+                                                  logger=self.cgmes_logger)
+                    data_parser.load_files(files=[self.file_name])
+
+                    if is_valid_cgmes(data_parser.cgmes_version):
+                        self.cgmes_circuit = CgmesCircuit(cgmes_version=data_parser.cgmes_version, text_func=text_func,
+                                                          progress_func=progress_func, logger=self.cgmes_logger)
+                        self.cgmes_circuit.parse_files(data_parser=data_parser)
+                        self.circuit = cgmes_to_gridcal(cgmes_model=self.cgmes_circuit, logger=self.cgmes_logger)
+                    else:
+                        # try CIM
+                        parser = CIMImport(text_func=text_func, progress_func=progress_func)
+                        self.circuit = parser.load_cim_file(self.file_name)
+                        self.logger += parser.logger
 
                 elif file_extension.lower() == '.hdf5':
                     self.circuit = parse_hdf5(self.file_name, self.logger)
@@ -266,33 +380,31 @@ class FileSave:
     FileSave
     """
 
-    def __init__(self, circuit: MultiCircuit, file_name, text_func=None, progress_func=None,
-                 simulation_drivers=list(), sessions=list(), json_files=dict()):
+    def __init__(self,
+                 circuit: MultiCircuit,
+                 file_name: str,
+                 options: FileSavingOptions = FileSavingOptions(),
+                 text_func=None,
+                 progress_func=None):
         """
         File saver
         :param circuit: MultiCircuit
         :param file_name: file name to save to
+        :param options: FileSavingOptions
         :param text_func: Pointer to the text function
         :param progress_func: Pointer to the progress function
-        :param simulation_drivers: List of Simulation Drivers
-        :param sessions: List of sessions
-        :param json_files: Dictionary of json files
         """
         self.circuit = circuit
 
         self.file_name = file_name
 
-        self.simulation_drivers = simulation_drivers
-
-        self.sessions = sessions
-
-        self.json_files = json_files
+        self.options = options
 
         self.text_func = text_func
 
         self.progress_func = progress_func
 
-    def save(self):
+    def save(self) -> Logger:
         """
         Save the file in the corresponding format
         :return: logger with information
@@ -300,7 +412,7 @@ class FileSave:
         if self.file_name.endswith('.xlsx'):
             logger = self.save_excel()
 
-        elif self.file_name.endswith('.gridcal'):
+        elif self.file_name.endswith('.gridcal') or self.file_name.endswith('.dgridcal'):
             logger = self.save_zip()
 
         elif self.file_name.endswith('.sqlite'):
@@ -309,7 +421,10 @@ class FileSave:
         elif self.file_name.endswith('.ejson3'):
             logger = self.save_json_v3()
 
-        elif self.file_name.endswith('.xml'):
+        elif self.file_name.endswith('.zip') and "CGMES" in self.options.type_selected:
+            logger = self.save_cgmes()
+
+        elif self.file_name.endswith('.xml') and "CIM" in self.options.type_selected:
             logger = self.save_cim()
 
         elif self.file_name.endswith('.gch5'):
@@ -333,7 +448,7 @@ class FileSave:
 
         return logger
 
-    def save_excel(self):
+    def save_excel(self) -> Logger:
         """
         Save the circuit information in excel format
         :return: logger with information
@@ -343,7 +458,7 @@ class FileSave:
 
         return logger
 
-    def save_zip(self):
+    def save_zip(self) -> Logger:
         """
         Save the circuit information in zip format
         :return: logger with information
@@ -351,19 +466,24 @@ class FileSave:
 
         logger = Logger()
 
-        dfs = create_data_frames(self.circuit)
+        dfs = gather_model_as_data_frames(self.circuit,
+                                          legacy=False)
 
-        save_data_frames_to_zip(dfs=dfs,
-                                filename_zip=self.file_name,
-                                sessions=self.sessions,
-                                diagrams=self.circuit.diagrams,
-                                json_files=self.json_files,
-                                text_func=self.text_func,
-                                progress_func=self.progress_func)
+        model_data = gather_model_as_jsons(self.circuit)
+
+        save_gridcal_data_to_zip(dfs=dfs,
+                                 filename_zip=self.file_name,
+                                 model_data=model_data,
+                                 sessions_data=self.options.sessions_data,
+                                 diagrams=self.circuit.diagrams,
+                                 json_files=self.options.dictionary_of_json_files,
+                                 text_func=self.text_func,
+                                 progress_func=self.progress_func,
+                                 logger=logger)
 
         return logger
 
-    def save_sqlite(self):
+    def save_sqlite(self) -> Logger:
         """
         Save the circuit information in sqlite
         :return: logger with information
@@ -371,7 +491,7 @@ class FileSave:
 
         logger = Logger()
 
-        dfs = create_data_frames(self.circuit)
+        dfs = gather_model_as_data_frames(self.circuit)
 
         save_data_frames_to_sqlite(dfs,
                                    file_path=self.file_name,
@@ -380,16 +500,18 @@ class FileSave:
 
         return logger
 
-    def save_json_v3(self):
+    def save_json_v3(self) -> Logger:
         """
         Save the circuit information in json format
         :return:logger with information
         """
 
-        logger = save_json_file_v3(self.file_name, self.circuit, self.simulation_drivers)
+        logger = save_json_file_v3(self.file_name,
+                                   self.circuit,
+                                   self.options.simulation_drivers)
         return logger
 
-    def save_cim(self):
+    def save_cim(self) -> Logger:
         """
         Save the circuit information in CIM format
         :return: logger with information
@@ -400,7 +522,46 @@ class FileSave:
 
         return cim.logger
 
-    def save_h5(self):
+    def save_cgmes(self) -> Logger:
+        """
+        Save the circuit information in CGMES format
+        :return: logger with information
+        """
+        logger = Logger()
+        if self.options.cgmes_boundary_set == "":
+            logger.add_error(msg="Missing Boundary set path.")
+            return logger
+        # CGMES version should be given in the settings
+        cgmes_circuit = CgmesCircuit(cgmes_version=self.options.cgmes_version,
+                                     text_func=self.text_func,
+                                     progress_func=self.progress_func, logger=logger)
+
+        data_parser = CgmesDataParser(text_func=self.text_func,
+                                      progress_func=self.progress_func,
+                                      logger=logger)
+        data_parser.load_files(files=[self.options.cgmes_boundary_set])
+        cgmes_circuit.parse_files(data_parser=data_parser)
+        profiles_to_export = self.options.cgmes_profiles
+        one_file_per_profile = self.options.one_file_per_profile
+        pf_results = self.options.get_power_flow_results()
+        cgmes_circuit = gridcal_to_cgmes(gc_model=self.circuit,
+                                         cgmes_model=cgmes_circuit,
+                                         pf_results=pf_results,
+                                         logger=logger)
+        cgmes_circuit = create_cgmes_headers(cgmes_model=cgmes_circuit,
+                                             profiles_to_export=profiles_to_export,
+                                             version="1",
+                                             desc="Test description.",
+                                             scenariotime="2021-02-09T19:30:00Z")
+
+        cim_exporter = CimExporter(cgmes_circuit=cgmes_circuit,
+                                   profiles_to_export=profiles_to_export,
+                                   one_file_per_profile=one_file_per_profile)
+        cim_exporter.export(self.file_name)
+
+        return logger
+
+    def save_h5(self) -> Logger:
         """
         Save the circuit information in CIM format
         :return: logger with information
@@ -412,30 +573,32 @@ class FileSave:
 
         return logger
 
-    def save_raw(self):
+    def save_raw(self) -> Logger:
         """
         Save the circuit information in json format
         :return:logger with information
         """
-        raw_circuit = gridcal_to_raw(self.circuit)
-        logger = write_raw(self.file_name, raw_circuit)
+        logger = Logger()
+        raw_circuit = gridcal_to_raw(self.circuit, logger=logger)
+        logger += write_raw(self.file_name, raw_circuit)
         return logger
 
-    def save_rawx(self):
+    def save_rawx(self) -> Logger:
         """
         Save the circuit information in json format
         :return:logger with information
         """
-        raw_circuit = gridcal_to_raw(self.circuit)
-        logger = write_rawx(self.file_name, raw_circuit)
+        logger = Logger()
+        raw_circuit = gridcal_to_raw(self.circuit, logger=logger)
+        logger += write_rawx(self.file_name, raw_circuit)
         return logger
 
-    def save_newton(self):
+    def save_newton(self) -> Logger:
         """
         Save the circuit information in sqlite
         :return: logger with information
         """
-        from GridCalEngine.Core.Compilers.circuit_to_newton_pa import to_newton_pa, npa
+        from GridCalEngine.Compilers.circuit_to_newton_pa import to_newton_pa, npa
         logger = Logger()
 
         time_series = self.circuit.time_profile is not None
@@ -451,12 +614,12 @@ class FileSave:
 
         return logger
 
-    def save_pgm(self):
+    def save_pgm(self) -> Logger:
         """
         Save to Power Grid Model format
         :return: logger with information
         """
-        from GridCalEngine.Core.Compilers.circuit_to_pgm import save_pgm
+        from GridCalEngine.Compilers.circuit_to_pgm import save_pgm
         logger = Logger()
 
         save_pgm(filename=self.file_name, circuit=self.circuit, logger=logger, time_series=self.circuit.has_time_series)
