@@ -1,5 +1,5 @@
 # GridCal
-# Copyright (C) 2015 - 2023 Santiago Peñate Vera
+# Copyright (C) 2015 - 2024 Santiago Peñate Vera
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,9 @@ import chardet
 import re
 from typing import List, AnyStr, Dict, Union
 
-from GridCalEngine.Core.Devices.multi_circuit import MultiCircuit
-import GridCalEngine.Core.Devices as dev
-import GridCalEngine.basic_structures as bs
+from GridCalEngine.Devices.multi_circuit import MultiCircuit
+import GridCalEngine.Devices as dev
+from GridCalEngine.basic_structures import Logger
 
 
 class PSSeObject:
@@ -85,7 +85,7 @@ class PSSeGrid:
         self.areas: List[PSSeArea] = list()
         self.zones: List[PSSeZone] = list()
 
-    def get_circuit(self, logger: bs.Logger, branch_connection_voltage_tolerance: float = 0.1) -> MultiCircuit:
+    def get_circuit(self, logger: Logger, branch_connection_voltage_tolerance: float = 0.1) -> MultiCircuit:
         """
         Returns GridCal circuit
         :param logger: Logger
@@ -147,6 +147,10 @@ class PSSeGrid:
             # psse_bus.bus.ensure_area_objects(circuit)
             circuit.add_bus(psse_bus.bus)
 
+            # in legacy PSSe buses there are sunts...so add them
+            if psse_bus.shunt is not None:
+                circuit.add_shunt(bus=psse_bus.bus, api_obj=psse_bus.shunt)
+
         for area in self.areas:
             if area.ISW not in slack_buses:
                 logger.add_error('The area slack bus is not marked as slack', str(area.ISW))
@@ -258,6 +262,9 @@ class PSSeBus(PSSeObject):
     def __init__(self):
         PSSeObject.__init__(self)
 
+        self.bus: Union[dev.Bus, None] = None
+        self.shunt: Union[dev.Shunt, None] = None
+
         self.I = 1
         self.NAME = ""
         self.BASKV = 1
@@ -272,7 +279,7 @@ class PSSeBus(PSSeObject):
         self.EVHI = 0
         self.EVLO = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -280,7 +287,7 @@ class PSSeBus(PSSeObject):
         :param logger:
         """
 
-        bustype = {1: dev.BusMode.PQ, 2: dev.BusMode.PV, 3: dev.BusMode.Slack, 4: dev.BusMode.PQ}
+        bustype = {1: dev.BusMode.PQ_tpe, 2: dev.BusMode.PV_tpe, 3: dev.BusMode.Slack_tpe, 4: dev.BusMode.PQ_tpe}
 
         if version >= 33:
             n = len(data[0])
@@ -293,7 +300,7 @@ class PSSeBus(PSSeObject):
             # create bus
             name = self.NAME.replace("'", "")
             self.bus = dev.Bus(name=name,
-                               vnom=self.BASKV, code=str(self.I), vmin=self.EVLO, vmax=self.EVHI, xpos=0, ypos=0,
+                               Vnom=self.BASKV, code=str(self.I), vmin=self.EVLO, vmax=self.EVHI, xpos=0, ypos=0,
                                active=True,
                                area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
@@ -303,7 +310,7 @@ class PSSeBus(PSSeObject):
 
             # create bus
             name = self.NAME
-            self.bus = dev.Bus(name=name, code=str(self.I), vnom=self.BASKV, vmin=self.NVLO, vmax=self.NVHI, xpos=0,
+            self.bus = dev.Bus(name=name, code=str(self.I), Vnom=self.BASKV, vmin=self.NVLO, vmax=self.NVHI, xpos=0,
                                ypos=0,
                                active=True, area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
@@ -314,15 +321,13 @@ class PSSeBus(PSSeObject):
 
             # create bus
             name = self.NAME
-            self.bus = dev.Bus(name=name, code=str(self.I), vnom=self.BASKV, vmin=0.9, vmax=1.1, xpos=0, ypos=0,
+            self.bus = dev.Bus(name=name, code=str(self.I), Vnom=self.BASKV, vmin=0.9, vmax=1.1, xpos=0, ypos=0,
                                active=True, area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
             if self.GL > 0 or self.BL > 0:
-                sh = dev.Shunt(name='Shunt_' + str(self.I),
-                               G=self.GL, B=self.BL,
-                               active=True)
-
-                self.bus.shunts.append(sh)
+                self.shunt = dev.Shunt(name='Shunt_' + str(self.I),
+                                       G=self.GL, B=self.BL,
+                                       active=True)
 
         else:
             logger.add_warning('Bus not implemented for version', str(version))
@@ -331,12 +336,12 @@ class PSSeBus(PSSeObject):
         if self.IDE in bustype.keys():
             self.bus.type = bustype[self.IDE]
         else:
-            self.bus.type = dev.BusMode.PQ
+            self.bus.type = dev.BusMode.PQ_tpe
 
         if int(self.IDE) == 4:
             self.bus.active = False
 
-        if self.bus.type == dev.BusMode.Slack:
+        if self.bus.type == dev.BusMode.Slack_tpe:
             self.bus.is_slack = True
 
         # Ensures unique name
@@ -373,7 +378,7 @@ class PSSeLoad(PSSeObject):
         self.DGENM = 0
         self.LOADTYPE = ''
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
 
         """
 
@@ -412,7 +417,7 @@ class PSSeLoad(PSSeObject):
         else:
             logger.add_warning('Load not implemented for version', str(version))
 
-    def get_object(self, bus: dev.Bus, logger: bs.Logger):
+    def get_object(self, bus: dev.Bus, logger: Logger):
         """
         Return Newton Load object
         Returns:
@@ -477,7 +482,7 @@ class PSSeSwitchedShunt(PSSeObject):
         self.RMIDNT = 0
         self.BINIT = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -501,7 +506,7 @@ class PSSeSwitchedShunt(PSSeObject):
         else:
             logger.add_warning('Shunt not implemented for the version', str(version))
 
-    def get_object(self, bus: dev.Bus, logger: bs.Logger):
+    def get_object(self, bus: dev.Bus, logger: Logger):
         """
         Return Newton Load object
         Returns:
@@ -542,7 +547,7 @@ class PSSeShunt(PSSeObject):
         self.GL = 0
         self.BL = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -554,7 +559,7 @@ class PSSeShunt(PSSeObject):
         else:
             logger.add_warning('Shunt not implemented for the version', str(version))
 
-    def get_object(self, bus: dev.Bus, logger: bs.Logger):
+    def get_object(self, bus: dev.Bus, logger: Logger):
         """
         Return Newton Load object
         Returns:
@@ -616,7 +621,7 @@ class PSSeGenerator(PSSeObject):
         self.WMOD = 0
         self.WPF = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -651,7 +656,7 @@ class PSSeGenerator(PSSeObject):
         else:
             logger.add_warning('Generator not implemented for version', str(version))
 
-    def get_object(self, logger: bs.Logger):
+    def get_object(self, logger: Logger):
         """
         Return Newton Load object
         Returns:
@@ -713,7 +718,7 @@ class PSSeInductionMachine(PSSeObject):
         self.IA2 = 0
         self.XAMULT = 1.0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -748,7 +753,7 @@ class PSSeInductionMachine(PSSeObject):
                             P=self.PSET,
                             vset=self.RATEKV,
                             Snom=self.MBASE,
-                            active=bool(self.STATUS))
+                            active=bool(self.STAT))
 
         return elm
 
@@ -801,7 +806,7 @@ class PSSeBranch(PSSeObject):
         self.MET = 0
         self.LEN = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -848,15 +853,15 @@ class PSSeBranch(PSSeObject):
 
             logger.add_warning('Branch not implemented for version', str(version))
 
-    def get_object(self, psse_bus_dict, Sbase, logger: bs.Logger):
+    def get_object(self, psse_bus_dict, Sbase, logger: Logger):
         """
         Return Newton branch object
-        Args:
-            psse_bus_dict: Dictionary that relates PSSe bus indices with Newton Bus objects
-
-        Returns:
-            Newton Branch object
+        :param psse_bus_dict: Dictionary that relates PSSe bus indices with Newton Bus objects
+        :param Sbase: Base power in MVA
+        :param logger: Logger
+        :return: Branch object
         """
+
         i = abs(self.I)
         j = abs(self.J)
         bus_from = psse_bus_dict[i]
@@ -946,7 +951,7 @@ class PSSeTwoTerminalDCLine(PSSeObject):
         self.IDI = 0
         self.XCAPI = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -990,7 +995,7 @@ class PSSeTwoTerminalDCLine(PSSeObject):
         else:
             logger.add_warning('Version not implemented for DC Lines', str(version))
 
-    def get_object(self, psse_bus_dict, Sbase, logger: bs.Logger):
+    def get_object(self, psse_bus_dict, Sbase, logger: Logger):
         """
         GEt equivalent object
         :param psse_bus_dict:
@@ -1089,7 +1094,7 @@ class PSSeVscDCLine(PSSeObject):
         self.REMOT2 = 0
         self.RMPCT2 = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -1133,7 +1138,7 @@ class PSSeVscDCLine(PSSeObject):
         else:
             logger.add_warning('Version not implemented for VSC-DC Lines', str(version))
 
-    def get_object(self, psse_bus_dict, Sbase, logger: bs.Logger):
+    def get_object(self, psse_bus_dict, Sbase, logger: Logger):
         """
         GEt equivalent object
         :param psse_bus_dict:
@@ -1390,7 +1395,7 @@ class PSSeTransformer(PSSeObject):
         self.RATE3_11 = 0
         self.RATE3_12 = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -1687,7 +1692,7 @@ class PSSeTransformer(PSSeObject):
         else:
             logger.add_warning('Transformer not implemented for version', str(version))
 
-    def get_object(self, psse_bus_dict, sbase, logger: bs.Logger):
+    def get_object(self, psse_bus_dict, sbase, logger: Logger):
         """
         Return Newton branch object
         Args:
@@ -1936,7 +1941,7 @@ class PSSeFACTS(PSSeObject):
         self.REMOT = 0
         self.MNAME = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -1978,7 +1983,7 @@ class PSSeFACTS(PSSeObject):
     def is_connected(self):
         return self.I > 0 and self.J > 0
 
-    def get_object(self, psse_bus_dict, Sbase, logger: bs.Logger, circuit: MultiCircuit):
+    def get_object(self, psse_bus_dict, Sbase, logger: Logger, circuit: MultiCircuit):
         """
         GEt equivalent object
         :param psse_bus_dict:
@@ -2087,7 +2092,7 @@ class PSSeInterArea(PSSeObject):
         self.PDES = 0
         self.PTOL = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -2115,7 +2120,7 @@ class PSSeArea(PSSeObject):
         self.PDES = 0
         self.PTOL = 0
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -2144,7 +2149,7 @@ class PSSeZone(PSSeObject):
         self.I = -1
         self.ZONAME = ''
 
-    def parse(self, data, version, logger: bs.Logger):
+    def parse(self, data, version, logger: Logger):
         """
 
         :param data:
@@ -2238,7 +2243,7 @@ class PSSeParser:
         self.parsers = dict()
         self.versions = [35, 34, 33, 32, 30, 29]
 
-        self.logger = bs.Logger()
+        self.logger = Logger()
 
         self.file_name = file_name
 
@@ -2394,7 +2399,7 @@ class PSSeParser:
         Returns: MultiCircuit, List[str]
         """
 
-        logger = bs.Logger()
+        logger = Logger()
 
         if text_func is not None:
             text_func("Reading file...")
