@@ -14,6 +14,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+from __future__ import annotations
+
 import os
 import chardet
 import datetime
@@ -42,7 +44,11 @@ from GridCalEngine.IO.raw.devices.psse_circuit import PsseCircuit
 
 
 def delete_comment(raw_line):
+    """
 
+    :param raw_line:
+    :return:
+    """
     lne = ""
     text_active = False
     for c in raw_line:
@@ -107,6 +113,7 @@ def interpret_line(raw_line, splitter=','):
 
     return parsed
 
+
 #
 # class PSSeRawParser:
 #     """
@@ -130,7 +137,7 @@ def interpret_line(raw_line, splitter=','):
 #
 #         self.logger += logs
 
-def read_and_split(file_name, text_func=None,  progress_func=None) -> (List[AnyStr], Dict[AnyStr, AnyStr]):
+def read_and_split(file_name, text_func=None, progress_func=None) -> (List[AnyStr], Dict[AnyStr, AnyStr]):
     """
     Read the text file and split it into sections
     :return: list of sections, dictionary of sections by type
@@ -168,7 +175,7 @@ def read_and_split(file_name, text_func=None,  progress_func=None) -> (List[AnyS
                     sections_dict[block_category] = list()
 
                 if i == 0:
-                    sections_dict['info'] = [interpret_line(raw_line=lne, splitter=sep)]
+                    sections_dict['info'] = [interpret_line(raw_line=lne, splitter=sep)]  # TODO: Fix the typing
                 elif i == 1:
                     sections_dict['comment'] = [lne]
                 elif i == 2:
@@ -191,7 +198,7 @@ def read_and_split(file_name, text_func=None,  progress_func=None) -> (List[AnyS
                         pass
                     else:
                         if lne.strip() != '':
-                            sections_dict[block_category].append(interpret_line(raw_line=lne, splitter=sep))
+                            sections_dict[block_category].append(interpret_line(raw_line=lne, splitter=sep))  # TODO: Fix the typing
 
                 i += 1
             else:
@@ -202,7 +209,7 @@ def read_and_split(file_name, text_func=None,  progress_func=None) -> (List[AnyS
     return sections_dict
 
 
-def read_raw(filename, text_func=None,  progress_func=None, logger = Logger()) -> PsseCircuit:
+def read_raw(filename, text_func=None, progress_func=None, logger=Logger()) -> PsseCircuit:
     """
 
     :param filename:
@@ -225,7 +232,7 @@ def read_raw(filename, text_func=None,  progress_func=None, logger = Logger()) -
     if text_func is not None:
         text_func("Reading file...")
 
-    sections_dict = read_and_split(file_name=filename, text_func=text_func,  progress_func=progress_func)
+    sections_dict = read_and_split(file_name=filename, text_func=text_func, progress_func=progress_func)
 
     # header -> new grid
     # grid = PSSeGrid(interpret_line(sections[0]))
@@ -298,6 +305,9 @@ def read_raw(filename, text_func=None,  progress_func=None, logger = Logger()) -
     def is_3w(row):
         return row[0] in bus_set and row[1] in bus_set and row[2] in bus_set
 
+    def is_one_line_for_induction_machine(row):
+        return len(row) != 12
+
     for key, lines in sections_dict.items():
 
         if key in meta_data:
@@ -317,17 +327,21 @@ def read_raw(filename, text_func=None,  progress_func=None, logger = Logger()) -
 
                     lines_per_object2 = lines_per_object
 
-                    if version in versions and key == 'transformer':
-                        # as you know the PSS/e raw format is nuts, that is why for v29 (onwards probably)
-                        # the transformers may have 4 or 5 lines to define them
-                        # so, to be able to know, we look at the line "l" and check if the first arguments
-                        # are 2 or 3 buses
-                        if is_3w(lines[l_count]):
-                            # 3 - windings
-                            lines_per_object2 = 5
-                        else:
-                            # 2-windings
-                            lines_per_object2 = 4
+                    if version in versions:
+                        if key == 'transformer':
+                            # as you know the PSS/e raw format is nuts, that is why for v29 (onwards probably)
+                            # the transformers may have 4 or 5 lines to define them
+                            # so, to be able to know, we look at the line "l" and check if the first arguments
+                            # are 2 or 3 buses
+                            if is_3w(lines[l_count]):
+                                # 3 - windings
+                                lines_per_object2 = 5
+                            else:
+                                # 2-windings
+                                lines_per_object2 = 4
+                        elif key == 'induction machine':
+                            if is_one_line_for_induction_machine(lines[l_count]):
+                                lines_per_object2 = 1
 
                     data = list()
                     for k in range(lines_per_object2):
@@ -367,7 +381,7 @@ def read_raw(filename, text_func=None,  progress_func=None, logger = Logger()) -
     return grid
 
 
-def write_raw(file_name: str, psse_model: PsseCircuit, version = 33) -> Logger:
+def write_raw(file_name: str, psse_model: PsseCircuit, version=33) -> Logger:
     """
     Write PsseCircuit as .raw version 33
     :param file_name: name of the file
