@@ -14,10 +14,10 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
+from __future__ import annotations
 import numpy as np
 import numba as nb
-
+from typing import Tuple, List, Union, TYPE_CHECKING
 from GridCalEngine.Devices.multi_circuit import MultiCircuit
 from GridCalEngine.DataStructures.numerical_circuit import compile_numerical_circuit_at
 from GridCalEngine.Simulations.LinearFactors.linear_analysis import LinearAnalysis
@@ -26,6 +26,10 @@ from GridCalEngine.Simulations.results_template import ResultsTemplate
 from GridCalEngine.Simulations.driver_template import DriverTemplate
 from GridCalEngine.Simulations.ATC.available_transfer_capacity_options import AvailableTransferCapacityOptions
 from GridCalEngine.enumerations import StudyResultsType, AvailableTransferMode, ResultTypes, DeviceType, SimulationTypes
+from GridCalEngine.basic_structures import Vec, IntVec, Mat
+
+if TYPE_CHECKING:
+    from GridCalEngine.Simulations import ClusteringResults
 
 
 @nb.njit()
@@ -148,100 +152,13 @@ def compute_alpha(ptdf, P0, Pgen, Pinstalled, Pload, idx1, idx2, dT=1.0, mode=0,
     return alpha
 
 
-#
-# @nb.njit()
-# def compute_atc(br_idx, contingency_br_idx, lodf, alpha, flows, rates, contingency_rates, threshold=0.005):
-#     """
-#     Compute all lines' ATC
-#     :param br_idx: array of branch indices to analyze
-#     :param contingency_br_idx: array of branch indices to fail
-#     :param lodf: Line outage distribution factors (n-branch, n-outage branch)
-#     :param alpha: Branch sensitivities to the exchange [p.u.]
-#     :param flows: Branches power injected at the "from" side [MW]
-#     :param rates: all Branches rates vector
-#     :param contingency_rates: all Branches contingency rates vector
-#     :param threshold: value that determines if a line is studied for the ATC calculation
-#     :return:
-#              beta_mat: Matrix of beta values (branch, contingency_branch)
-#              beta: vector of actual beta value used for each branch (n-branch)
-#              atc_n: vector of ATC values in "N" (n-branch)
-#              atc_final: vector of ATC in "N" or "N-1" whatever is more limiting (n-branch)
-#              atc_limiting_contingency_branch: most limiting contingency branch index vector (n-branch)
-#              atc_limiting_contingency_flow: most limiting contingency flow vector (n-branch)
-#     """
-#
-#     nbr = len(br_idx)
-#
-#     # explore the ATC
-#     atc_n = np.zeros(nbr)
-#     atc_mc = np.zeros(nbr)
-#     atc_final = np.zeros(nbr)
-#     beta_mat = np.zeros((nbr, nbr))
-#     beta_used = np.zeros(nbr)
-#     atc_limiting_contingency_branch = np.zeros(nbr)
-#     atc_limiting_contingency_flow = np.zeros(nbr)
-#     # processed = list()
-#     # mm = 0
-#     for im, m in enumerate(br_idx):  # for each branch
-#
-#         # if abs(alpha[m]) > threshold and abs(flows[m]) < rates[m]:  # if the branch is relevant enough for the ATC...
-#         if abs(alpha[m]) > threshold:  # if the branch is relevant enough for the ATC...
-#
-#             # compute the ATC in "N"
-#             if alpha[m] == 0:
-#                 atc_final[im] = np.inf
-#             elif alpha[m] > 0:
-#                 atc_final[im] = (rates[m] - flows[m]) / alpha[m]
-#             else:
-#                 atc_final[im] = (-rates[m] - flows[m]) / alpha[m]
-#
-#             # remember the ATC in "N"
-#             atc_n[im] = atc_final[im]
-#
-#             # set to the current branch, since we don't know if there will be any contingency that make the ATC worse
-#             atc_limiting_contingency_branch[im] = m
-#
-#             # explore the ATC in "N-1"
-#             for ic, c in enumerate(contingency_br_idx):  # for each contingency
-#                 # compute the exchange sensitivity in contingency conditions
-#                 beta_mat[im, ic] = alpha[m] + lodf[m, c] * alpha[c]
-#
-#                 if m != c:
-#
-#                     # compute the contingency flow
-#                     contingency_flow = flows[m] + lodf[m, c] * flows[c]
-#
-#                     # set the default values (worst contingency by itself, not comparing with the base situation)
-#                     if abs(contingency_flow) > abs(atc_limiting_contingency_flow[im]):
-#                         atc_limiting_contingency_flow[im] = contingency_flow  # default
-#                         atc_limiting_contingency_branch[im] = c
-#
-#                     # now here, do compare with the base situation
-#                     if abs(beta_mat[im, ic]) > threshold and abs(contingency_flow) <= contingency_rates[m]:
-#
-#                         # compute the ATC in "N-1"
-#                         if beta_mat[im, ic] == 0:
-#                             atc_mc[im] = np.inf
-#                         elif beta_mat[im, ic] > 0:
-#                             atc_mc[im] = (contingency_rates[m] - contingency_flow) / beta_mat[im, ic]
-#                         else:
-#                             atc_mc[im] = (-contingency_rates[m] - contingency_flow) / beta_mat[im, ic]
-#
-#                         # refine the ATC to the most restrictive value every time
-#                         if abs(atc_mc[im]) < abs(atc_final[im]):
-#                             atc_final[im] = atc_mc[im]
-#                             beta_used[im] = beta_mat[im, ic]
-#                             atc_limiting_contingency_flow[im] = contingency_flow
-#                             atc_limiting_contingency_branch[im] = c
-#
-#     return beta_mat, beta_used, atc_n, atc_mc, atc_final, atc_limiting_contingency_branch, atc_limiting_contingency_flow
-
-
 @nb.njit()
-def compute_atc_list(br_idx, contingency_br_idx, lodf, alpha, flows, rates, contingency_rates, base_exchange,
-                     threshold, time_idx):
+def compute_atc_list(br_idx: IntVec, contingency_br_idx: IntVec, lodf: Mat, alpha: Vec, flows: Vec, rates: Vec,
+                     contingency_rates: Vec, base_exchange: float, threshold: float,
+                     time_idx: int) -> List[
+    Tuple[int, int, int, float, float, float, float, float, float, float, float, float, float, float, float]]:
     """
-    Compute all lines' ATC
+    Compute all lines' available transfer capacity (ATC)
     :param br_idx: array of branch indices to analyze
     :param contingency_br_idx: array of branch indices to fail
     :param lodf: Line outage distribution factors (n-branch, n-outage branch)
@@ -252,13 +169,22 @@ def compute_atc_list(br_idx, contingency_br_idx, lodf, alpha, flows, rates, cont
     :param base_exchange: amount already exchanges between areas
     :param threshold: value that determines if a line is studied for the ATC calculation
     :param time_idx: time index of the calculation
-    :return:
-             beta_mat: Matrix of beta values (branch, contingency_branch)
-             beta: vector of actual beta value used for each branch (n-branch)
-             atc_n: vector of ATC values in "N" (n-branch)
-             atc_final: vector of ATC in "N" or "N-1" whatever is more limiting (n-branch)
-             atc_limiting_contingency_branch: most limiting contingency branch index vector (n-branch)
-             atc_limiting_contingency_flow: most limiting contingency flow vector (n-branch)
+    :return: List of:
+        time_idx,  # 0
+        monitored index,  # 1
+        contingency index,  # 2
+        alpha of branch m,  # 3
+        beta,  # 4
+        lodf[m, c],  # 5
+        atc_n,  # 6
+        atc_mc,  # 7
+        final_atc,  # 8
+        ntc,  # 9
+        flows[m],  # 10
+        contingency_flow,  # 11
+        loading,  # 12
+        contingency loading,  # 13
+        base_exchange #14
     """
 
     results = list()
@@ -323,11 +249,15 @@ def compute_atc_list(br_idx, contingency_br_idx, lodf, alpha, flows, rates, cont
 
 class AvailableTransferCapacityResults(ResultsTemplate):
 
-    def __init__(self, br_names, bus_names, rates, contingency_rates, clustering_results):
+    def __init__(self, br_names, bus_names, rates, contingency_rates: Vec,
+                 clustering_results: Union[ClusteringResults, None]):
         """
 
         :param br_names:
         :param bus_names:
+        :param rates:
+        :param contingency_rates:
+        :param clustering_results:
         """
         ResultsTemplate.__init__(self,
                                  name='ATC Results',
@@ -349,6 +279,10 @@ class AvailableTransferCapacityResults(ResultsTemplate):
         self.raw_report = None
 
     def get_steps(self):
+        """
+
+        :return: 
+        """
         return
 
     def make_report(self, threshold: float = 0.0):
@@ -452,7 +386,7 @@ class AvailableTransferCapacityResults(ResultsTemplate):
         if result_type == ResultTypes.AvailableTransferCapacityReport:
             data = np.array(self.report)
             y_label = ''
-            title, _ = result_type.value
+            title = result_type.value
 
             return ResultsTable(data=np.array(self.report),
                                 index=self.report_indices,
@@ -488,7 +422,7 @@ class AvailableTransferCapacityDriver(DriverTemplate):
                                                         contingency_rates=[],
                                                         clustering_results=None)
 
-    def run(self):
+    def run(self) -> None:
         """
         Run thread
         """
@@ -599,7 +533,7 @@ class AvailableTransferCapacityDriver(DriverTemplate):
 
         self.toc()
 
-    def get_steps(self):
+    def get_steps(self) -> List[int]:
         """
         Get variations list of strings
         """
