@@ -159,8 +159,7 @@ class NodalCapacityTimeSeriesDriver(TimeSeriesDriverTemplate):
                                      all_generators_fixed=False,
                                      lodf_threshold=self.opf_options.lodf_tolerance,
                                      maximize_inter_area_flow=self.opf_options.maximize_flows,
-                                     areas_from=self.opf_options.areas_from,
-                                     areas_to=self.opf_options.areas_to,
+                                     inter_aggregation_info=self.opf_options.inter_aggregation_info,
                                      energy_0=batteries_energy_0,
                                      optimize_nodal_capacity=True,
                                      nodal_capacity_sign=self.options.nodal_capacity_sign,
