@@ -148,8 +148,7 @@ class OptimalPowerFlowDriver(TimeSeriesDriverTemplate):
                                          all_generators_fixed=False,
                                          lodf_threshold=self.options.lodf_tolerance,
                                          maximize_inter_area_flow=self.options.maximize_flows,
-                                         areas_from=self.options.areas_from,
-                                         areas_to=self.options.areas_to,
+                                         inter_aggregation_info=self.options.inter_aggregation_info,
                                          energy_0=None,
                                          fluid_level_0=None,
                                          logger=self.logger,
