@@ -155,11 +155,7 @@ def short_circuit_unbalanced(calculation_inputs: NumericalCircuit,
                                Beq=np.zeros(nbr),
                                Cf=calculation_inputs.branch_data.C_branch_bus_f.tocsc(),
                                Ct=calculation_inputs.branch_data.C_branch_bus_t.tocsc(),
-                               G0sw=np.zeros(nbr),
-                               If=np.zeros(nbr),
-                               a=np.zeros(nbr),
-                               b=np.zeros(nbr),
-                               c=np.zeros(nbr),
+                               Gsw=np.zeros(nbr),
                                Yshunt_bus=Yshunt_bus0,
                                conn=calculation_inputs.branch_data.conn,
                                seq=0,
@@ -181,11 +177,7 @@ def short_circuit_unbalanced(calculation_inputs: NumericalCircuit,
                                Beq=calculation_inputs.branch_data.Beq,
                                Cf=calculation_inputs.branch_data.C_branch_bus_f.tocsc(),
                                Ct=calculation_inputs.branch_data.C_branch_bus_t.tocsc(),
-                               G0sw=calculation_inputs.branch_data.G0sw,
-                               If=np.zeros(nbr),
-                               a=calculation_inputs.branch_data.a,
-                               b=calculation_inputs.branch_data.b,
-                               c=calculation_inputs.branch_data.c,
+                               Gsw=calculation_inputs.branch_data.G0sw,
                                Yshunt_bus=Yshunt_bus1,
                                conn=calculation_inputs.branch_data.conn,
                                seq=1,
@@ -207,11 +199,7 @@ def short_circuit_unbalanced(calculation_inputs: NumericalCircuit,
                                Beq=np.zeros(nbr),
                                Cf=calculation_inputs.branch_data.C_branch_bus_f.tocsc(),
                                Ct=calculation_inputs.branch_data.C_branch_bus_t.tocsc(),
-                               G0sw=np.zeros(nbr),
-                               If=np.zeros(nbr),
-                               a=np.zeros(nbr),
-                               b=np.zeros(nbr),
-                               c=np.zeros(nbr),
+                               Gsw=np.zeros(nbr),
                                Yshunt_bus=Yshunt_bus2,
                                conn=calculation_inputs.branch_data.conn,
                                seq=2,
@@ -251,11 +239,7 @@ def short_circuit_unbalanced(calculation_inputs: NumericalCircuit,
                                      Beq=np.zeros(nbr),
                                      Cf=calculation_inputs.branch_data.C_branch_bus_f.tocsc(),
                                      Ct=calculation_inputs.branch_data.C_branch_bus_t.tocsc(),
-                                     G0sw=np.zeros(nbr),
-                                     If=np.zeros(nbr),
-                                     a=calculation_inputs.branch_data.a,
-                                     b=calculation_inputs.branch_data.b,
-                                     c=calculation_inputs.branch_data.c,
+                                     Gsw=np.zeros(nbr),
                                      Yshunt_bus=np.zeros(nbus, dtype=complex),
                                      conn=calculation_inputs.branch_data.conn,
                                      seq=1,
